/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/

package casa.lepton.hub;

public class SDNV {

    // Implementation according to
    // https://www.rfc-editor.org/info/rfc6256
    
    // ------------------------------------------------------------
    // Return the number of bytes required to encode v as an SDNV //
    // (Self-Delimiting Numeric Value, RFC 6256)
    public static int encodingLength(long v) {
	
	int length = 0;
	do {
	    v = v >> 7;
	    length++;
	    if (v == 0)
		return length;
	} while (v != 0);
	return length;
    }

    // ------------------------------------------------------------
    // Decode and return the value encoded in the buffer as an SDNV
    public static long decode(byte[] data, int offset) {

	long result = 0;
	try {
	    byte cursor = 0;
	    do {
		cursor = data[offset];
		offset++;
		result = result << 7;
		result += cursor & 0x7F;
	    } while ((cursor & 0x80) != 0);
	}
	catch (Exception e) {
	    return -1;
	}
	return result;
    }

     // ------------------------------------------------------------
    // Encode v as an SDNV, put the results in the byte array, and
    // return the number of bytes required for this encoding
    public static int encode(long v, byte[] data, int offset) {

	int length = encodingLength(v);
	int idx = length;
	do {
	    byte low = (byte)(v & 0x7F);
	    v = v >> 7;
	    if (v != 0)
		low = (byte)-low; // bytes are signed in Java
	    data[offset+idx-1] = low;
	    idx--;
	} while (idx > 0);
	return length;
    }

}
